from PyQt4 import QtCore, QtGui

class SystemMonitor(QtGui.QWidget):
  def __init__(self, parent):
    QtGui.QWidget.__init__(self, parent)
    (self.prev_cpu_values, self.prev_cpu_sum)=self.collectData()

    self.update_timer=QtCore.QTimer()
    self.connect(self.update_timer, QtCore.SIGNAL("timeout()"), self.refresh)
    self.update_timer.start(1000)

  def refresh(self):
    self.update()

  def paintEvent(self, e):
    (cpu, ram, swap)=self.getSystemInfo()

    w=self.width()/5
    h=self.height()/2
    h_cpu=h*cpu/100
    h_ram=h*ram/100
    h_swap=h*swap/100
    h_mid=h+h/2
    painter=QtGui.QPainter(self)
    painter.fillRect(w, h_mid-h_cpu, w, h_cpu, QtGui.QBrush(QtCore.Qt.red))
    painter.fillRect(w*2, h_mid-h_ram, w, h_ram, QtGui.QBrush(QtCore.Qt.blue))
    painter.fillRect(w*3, h_mid-h_swap, w, h_swap, QtGui.QBrush(QtCore.Qt.green))
    painter.end()

  def resizeEvent(self, e):
    w=e.size().height()
    self.resize(w, e.size().height())
    self.setMinimumWidth(w)

  def valueToInt(self, value):
    return int(value.strip().split(" ")[0])

  def collectData(self):
    values=map(int, open("/proc/stat", "r").readline().split(" ")[2:])
    sum=reduce(lambda x,y: x+y, values)
    return (values, sum)
    
  def getSystemInfo(self):
    """Compute system statistics and return a tuple (%cpu, %ram, %swap)."""
    (values, sum)=self.collectData()

    # number of USER_HZ elapsed
    delta=sum-self.prev_cpu_sum

    # number of USER_HZ, minus time spent in idle, converted as a percentage
    if delta!=0:
      cpu=(sum-self.prev_cpu_sum-(values[3]-self.prev_cpu_values[3]))*100/delta
    else:
      cpu=100
    
    self.prev_cpu_values=values
    self.prev_cpu_sum=sum

    # end when the last value SwapFree is found
    mem=open("/proc/meminfo", "r")
    while 1:
        (key, value)=mem.readline().split(":")
        if key=="MemTotal":
            mem_total=self.valueToInt(value)
        elif key=="MemFree":
            mem_free=self.valueToInt(value)
        elif key=="Buffers":
            buffers=self.valueToInt(value)
        elif key=="Cached":
            cached=self.valueToInt(value)
        elif key=="SwapTotal":
            swap_total=self.valueToInt(value)
        elif key=="SwapFree":
            swap_free=self.valueToInt(value)
            break

    ram=(mem_total-mem_free-buffers-cached)*100/mem_total
    if swap_total:
      swap=(swap_total-swap_free)*100/swap_total
    else:
      swap=0

    return (cpu, ram, swap)
