import os, sys
from PyQt4 import QtCore, QtGui

from core.auimd_widget import *



class MainMenu(QtGui.QStackedWidget, AuimdWidget):
  @staticmethod
  def defaultWidgetIcon():
    return QtGui.QIcon("core/widgets/icons/taskbar.png")

  @staticmethod
  def defaultWidgetTitle():
    return "Main menu"

  def __init__(self, parent):
    QtGui.QStackedWidget.__init__(self, parent)
    AuimdWidget.__init__(self, parent)

    self.main_menu=QtGui.QWidget(self)
    self.addWidget(self.main_menu)

    self.main_menu.layout=QtGui.QGridLayout(self.main_menu)
    self.main_menu.layout.setMargin(0)
    self.main_menu.layout.setSpacing(0)

    s=self.desktop.size()
    if s.width()<=240 or s.height()<=240:
      icon_size=QtCore.QSize(48, 48)
    else:
      icon_size=QtCore.QSize(64, 64)
      

    self.bt_graphics   = self.addMainButton("core/widgets/icons/mainmenu/colors.png", "Graphics", 0, 0, icon_size)
    self.bt_games      = self.addMainButton("core/widgets/icons/mainmenu/package_games.png", "Games", 0, 1, icon_size)
    self.bt_multimedia = self.addMainButton("core/widgets/icons/mainmenu/package_multimedia.png", "Multimedia", 0, 2, icon_size)
    self.bt_tools      = self.addMainButton("core/widgets/icons/mainmenu/package_utilities.png", "Tools", 1, 0, icon_size)
    self.bt_network    = self.addMainButton("core/widgets/icons/mainmenu/package_network.png", "Network", 1, 1, icon_size)
    self.bt_pim        = self.addMainButton("core/widgets/icons/mainmenu/date.png", "PIM", 1, 2, icon_size)
    self.bt_settings   = self.addMainButton("core/widgets/icons/mainmenu/package_settings.png", "Settings", 2, 0, icon_size)
    self.bt_system     = self.addMainButton("core/widgets/icons/mainmenu/terminal.png", "System", 2, 1, icon_size)
    self.bt_other      = self.addMainButton("core/widgets/icons/mainmenu/package_favourite.png", "Other", 2, 2, icon_size)

    self.bt_graphics.category   = "Graphics"
    self.bt_games.category      = "Games"
    self.bt_multimedia.category = "Multimedia"
    self.bt_tools.category      = "Tools"
    self.bt_network.category    = "Network"
    self.bt_pim.category        = "PIM"
    self.bt_settings.category   = "Settings"
    self.bt_system.category     = "System"
    self.bt_other.category      = "Other"

    self.sub_menu=QtGui.QListWidget(self)
    self.addWidget(self.sub_menu)
    self.sub_menu.setViewMode(QtGui.QListView.IconMode)
    self.sub_menu.setMovement(QtGui.QListView.Static)
    QtCore.QObject.connect(self.sub_menu, QtCore.SIGNAL("itemClicked(QListWidgetItem*)"), self.subMenuItemActivated)

    self.setAutoFillBackground(1)

    self.setButtonsText()

  def addMainButton(self, icon, text, row, column, icon_size):
    b=MainMenuButton(self.main_menu, icon, text, self.mainButtonPressed, icon_size)
    self.main_menu.layout.addWidget(b, row, column)
    return b

  def setButtonsText(self):
    if self.currentIndex()==0:
      self.left_button.setText("Reload")
      self.right_button.setText("Close")
    else:
      self.left_button.setText("Back")
      self.right_button.setText("Close")

  def leftButtonAction(self):
    if self.currentIndex()==1:
      self.setCurrentIndex(0)
      self.setButtonsText()
    else:
        self.desktop.reloadWidgets()

  def rightButtonAction(self):
    self.desktop.closeWidget(self)

  def mainButtonPressed(self, button):
    self.sub_menu.clear()
    for classs in self.desktop.widget_classes[button.category]:
      self.createSubMenuItem(classs)

    self.left_button.setText("Back")
    self.setCurrentIndex(1)

  def createSubMenuItem(self, classs):
    item=QtGui.QListWidgetItem(self.sub_menu)
    item.setText(classs.defaultWidgetTitle())
    item.setIcon(classs.defaultWidgetIcon())
    item.class_name=classs.__name__

  def subMenuItemActivated(self, item):
    self.desktop.startWidget(item.class_name)

  def keyboardAllowed(self):
    return 0

class MainMenuButton(QtGui.QToolButton):
  def __init__(self, parent, icon, text, action, icon_size):
    QtGui.QToolButton.__init__(self, parent)
    self.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
    self.setIconSize(icon_size)
    self.setIcon(QtGui.QIcon(icon))
    self.setText(text)
    self.setFocusPolicy(QtCore.Qt.NoFocus)
    self.setMaximumSize(130,130)
    self.action=action

    QtCore.QObject.connect(self, QtCore.SIGNAL("clicked()"), self.buttonClicked)

  def buttonClicked(self):
    self.action(self)
