import os
from devices.common.base_device import *

class Device(BaseDevice):
  def __init__(self):
    self.bl_base="/sys/class/backlight/corgi-bl/"
    self.bat_base="/sys/devices/w1_bus_master1/30-00001264e43e/ds2760-battery.0/power_supply:ds2760-battery.0/"

    if not os.path.exists(self.bl_base+"max_brightness"):
      self.has_screen_brightness=0
      self.has_screen_power_control=0
    else:
      self.has_screen_brightness=1
      self.has_screen_power_control=1
      self.max_brightness=self.readValueFromFile(self.bl_base+"max_brightness")

    if not os.path.exists(self.bat_base+"charge_full"):
      self.has_battery_status=0
    else:
      self.has_battery_status=1
      self.bat_charge_full=self.readValueFromFile(self.bat_base+"charge_full")

  def hasScreenBrightnessControl(self):
    return self.has_screen_brightness

  def hasScreenPowerControl(self):
    return self.has_screen_power_control
    
  def setScreenBrightness(self, val):
    v=val*self.max_brightness/100
    self.writeValueToFile(self.bl_base+"brightness", v)

  def getScreenBrightness(self):
    v=self.readValueFromFile(self.bl_base+"brightness")
    return v*100/self.max_brightness

  def setScreenPower(self, power):
    # for the sysfs file : 0=on, 1=off
    if power: v=0
    else: v=1
    self.writeValueToFile(self.bl_base+"power", v)

  def getScreenPower(self):
    # for the sysfs file : 0=on, 1=off
    return not self.readValueFromFile(self.bl_base+"power")

  def hasBatteryStatus(self):
    return self.has_battery_status

  def getBatteryStatus(self):
    v=self.readValueFromFile(self.bat_base+"charge_now")
    if open(self.bat_base+"status", "r").readline().strip()=="Charging":
      charging=1
    else:
      charging=0
    return (v*100/self.bat_charge_full, charging)
