#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>

static void *lib_xlib = NULL;

static void OutputWindowId(Display *display, Window window)
{
  fprintf(stderr, "PE_X11_NEW_WINDOW %d\n", (int)window);
}

/* XCreateWindow intercept hack */
Window
XCreateWindow(
              Display *display,
              Window parent,
              int x, int y,
              unsigned int width, unsigned int height,
              unsigned int border_width,
              int depth,
              unsigned int class,
              Visual *visual,
              unsigned long valuemask,
              XSetWindowAttributes *attributes
              )
{
   static Window (*func)
      (
       Display *display,
       Window parent,
       int x, int y,
       unsigned int width, unsigned int height,
       unsigned int border_width,
       int depth,
       unsigned int class,
       Visual *visual,
       unsigned long valuemask,
       XSetWindowAttributes *attributes
       ) = NULL;
   int i;

   /* find the real Xlib and the real X function */
   if (!lib_xlib) lib_xlib = dlopen("libX11.so", RTLD_GLOBAL | RTLD_LAZY);
   if (!func) func = dlsym (lib_xlib, "XCreateWindow");

   /* multihead screen handling loop */
   for (i = 0; i < ScreenCount(display); i++)
     {
        /* if the window is created as a toplevel window */
        if (parent == RootWindow(display, i))
          {
        printf("XCreateWindow %d %d\n", (int)parent, (int)RootWindow(display, i));
             Window window;
             
             /* create it */
             attributes->override_redirect = True;
             valuemask |= CWOverrideRedirect;
             window = (*func) (display, parent, x, y, width, height, 
                                border_width, depth, class, visual, valuemask, 
                                attributes);
             OutputWindowId(display, window);
             return window;
          }
     }
   /* normal child window - create as usual */
   return (*func) (display, parent, x, y, width, height, border_width, depth,
                   class, visual, valuemask, attributes);
}

/* XCreateSimpleWindow intercept hack */
Window
XCreateSimpleWindow(
                    Display *display,
                    Window parent,
                    int x, int y,
                    unsigned int width, unsigned int height,
                    unsigned int border_width,
                    unsigned long border,
                    unsigned long background
                    )
{
   static Window (*func)
      (
       Display *display,
       Window parent,
       int x, int y,
       unsigned int width, unsigned int height,
       unsigned int border_width,
       unsigned long border,
       unsigned long background
       ) = NULL;
   int i;
   
   /* find the real Xlib and the real X function */
   if (!lib_xlib) lib_xlib = dlopen("libX11.so", RTLD_GLOBAL | RTLD_LAZY);
   if (!func) func = dlsym (lib_xlib, "XCreateSimpleWindow");
   
   /* multihead screen handling loop */
   for (i = 0; i < ScreenCount(display); i++)
     {
        /* if the window is created as a toplevel window */
        if (parent == RootWindow(display, i))
          {
        printf("XCreateSimpleWindow %d %d\n", (int)parent, (int)RootWindow(display, i));
             Window window;
             XSetWindowAttributes attributes;
             
             /* create it */
             window = (*func) (display, parent, x, y, width, height, 
                                border_width, border, background);
             attributes.override_redirect = True;
             XChangeWindowAttributes(display, window, CWOverrideRedirect, &attributes);
             OutputWindowId(display, window);
             return window;
          }
     }
   /* normal child window - create as usual */
   return (*func) (display, parent, x, y, width, height, 
                   border_width, border, background);
}

#if 1
/* XReparentWindow intercept hack */
int
XReparentWindow(
                Display *display,
                Window window,
                Window parent,
                int x, int y
                )
{
   static int (*func)
      (
       Display *display,
       Window window,
       Window parent,
       int x, int y
       ) = NULL;
   int i;
   int res;
   
   /* find the real Xlib and the real X function */
   if (!lib_xlib) lib_xlib = dlopen("libX11.so", RTLD_GLOBAL | RTLD_LAZY);
   if (!func) func = dlsym (lib_xlib, "XReparentWindow");
   
   /* multihead screen handling loop */
   for (i = 0; i < ScreenCount(display); i++)
     {
        /* if the window is created as a toplevel window */
        printf("XReparentWindow %d %d\n", (int)parent, (int)RootWindow(display, i));
        if (parent == RootWindow(display, i))
          {
             XSetWindowAttributes attributes;
             OutputWindowId(display, window);
             res=(*func) (display, window, parent, x, y);
             attributes.override_redirect = True;
             XChangeWindowAttributes(display, window, CWOverrideRedirect, &attributes);
             return res;
          }
     }
   /* normal child window reparenting - reparent as usual */
   return (*func) (display, window, parent, x, y);
}
#endif

/*int XMapWindow(Display *display, Window window)
{
  static int (*func)(Display *display, Window window) = NULL;
  int i, res;
  unsigned nchildren;
  Window root, parent, *children;

  if (!func) func = dlsym (lib_xlib, "XMapWindow");

  res=func(display, window);

  if(XQueryTree(display, window, &root, &parent, &children, &nchildren)) {
    for (i = 0; i < ScreenCount(display); i++) {
      if (parent == RootWindow(display, i)) {
        OutputWindowId(display, window);
      }
    }
  }

  return res;
}*/
