from PyQt4 import QtCore, QtGui, uic

from . import __path__
from core.auimd_widget import AuimdWidget
from keyboard import Keyboard

(Ui_IncomingCall, IncomingCallBaseClass) = uic.loadUiType(__path__[0]+"/ui/incoming_call.ui")
(Ui_OutgoingCall, OutgoingCallBaseClass) = uic.loadUiType(__path__[0]+"/ui/outgoing_call.ui")
(Ui_CallInProgress, CallInProgressBaseClass) = uic.loadUiType(__path__[0]+"/ui/call_in_progress.ui")

class CallInProgress(CallInProgressBaseClass, Ui_CallInProgress):
  def __init__(self, parent):
    CallInProgressBaseClass.__init__(self, parent)
    self.setupUi(self)

class OutgoingCall(OutgoingCallBaseClass, Ui_OutgoingCall):
  def __init__(self, parent):
    OutgoingCallBaseClass.__init__(self, parent)
    self.setupUi(self)

class IncomingCall(IncomingCallBaseClass, Ui_IncomingCall):
  def __init__(self, parent):
    IncomingCallBaseClass.__init__(self, parent)
    self.setupUi(self)

class DtmfKeyboard(QtGui.QWidget):
  def __init__(self, parent):
    QtGui.QWidget.__init__(self, parent)
    l=QtGui.QVBoxLayout(self)

    self.keyboard=Keyboard(self, Keyboard.MODE_DTMF)
    self.back=QtGui.QPushButton(QtGui.QIcon(__path__[0]+"/icons/back.png"), "Back", self)
    self.back.setIconSize(QtCore.QSize(32, 32))
    l.addWidget(self.keyboard)
    l.addWidget(self.back)

class CallUi(QtGui.QStackedWidget, AuimdWidget):
  @staticmethod
  def defaultWidgetIcon():
    return QtGui.QIcon(__path__[0]+"/icons/voicecall.png")

  def __init__(self, parent, call):
    QtGui.QStackedWidget.__init__(self, parent)
    AuimdWidget.__init__(self, parent)
  
    self.call=call
    self.duration_timer=None
    self.in_call=True

    name=call.name
    if not name:
      name=call.phone_number
    # display the first screen depending on the call type (incoming, outgoing)
    if call.type==call.INCOMING:
      self.incoming=IncomingCall(self)
      self.addWidget(self.incoming)
      self.incoming.who.setText(name)
      self.connect(self.incoming.accept_call, QtCore.SIGNAL("clicked()"), self.onAcceptCallClicked)
      self.connect(self.incoming.reject_call, QtCore.SIGNAL("clicked()"), self.onRejectCallClicked)
      self.setCurrentWidget(self.incoming)
      self.setWidgetTitle("Incoming call : "+name)
    else:
      self.outgoing=OutgoingCall(self)
      self.addWidget(self.outgoing)
      self.outgoing.who.setText(name)
      self.connect(self.outgoing.cancel, QtCore.SIGNAL("clicked()"), self.onCancelClicked)
      self.setCurrentWidget(self.outgoing)
      self.setWidgetTitle("Outgoing call : "+name)

    # the call in progress screen is shown after the incoming or outgoing call screen
    self.call_in_progress=CallInProgress(self)
    self.call_in_progress.who.setText(name)
    self.addWidget(self.call_in_progress)
    self.connect(self.call_in_progress.hangup, QtCore.SIGNAL("clicked()"), self.onHangupClicked)
    self.connect(self.call_in_progress.speaker, QtCore.SIGNAL("clicked()"), self.onSpeakerClicked)
    self.connect(self.call_in_progress.keyboard, QtCore.SIGNAL("clicked()"), self.onKeyboardClicked)

    # the DTMF keyboard can be displayed from the call in progress
    self.dtmf_keyboard=DtmfKeyboard(self)
    self.addWidget(self.dtmf_keyboard)
    self.connect(self.dtmf_keyboard.back, QtCore.SIGNAL("clicked()"), self.onDtmfBackClicked)
    self.connect(self.dtmf_keyboard.keyboard, QtCore.SIGNAL("keyPressed"), self.onDtmfKeyPressed)

    # FIXME not implemented yet
    self.call_in_progress.speaker.setEnabled(False)

  def keyboardAllowed(self):
    return False

  def onAcceptCallClicked(self):
    self.emit(QtCore.SIGNAL("callAccepted"), self.call)
    self.incoming.accept_call.setEnabled(False)
    self.incoming.reject_call.setEnabled(False)

  def onRejectCallClicked(self):
    self.in_call=False
    self.emit(QtCore.SIGNAL("callRejected"), self.call)
    self.incoming.accept_call.setEnabled(False)
    self.incoming.reject_call.setEnabled(False)

  def onHangupClicked(self):
    self.in_call=False
    self.emit(QtCore.SIGNAL("callEnded"), self.call)
    self.call_in_progress.hangup.setEnabled(False)

  def onCancelClicked(self):
    self.in_call=False
    self.emit(QtCore.SIGNAL("callCanceled"), self.call)
    self.outgoing.cancel.setEnabled(False)

  def onSpeakerClicked(self):
    # TODO
    pass
 
  def onKeyboardClicked(self):
    self.setCurrentWidget(self.dtmf_keyboard)

  def onDtmfBackClicked(self):
    self.setCurrentWidget(self.call_in_progress)

  def onDtmfKeyPressed(self, k):
    self.emit(QtCore.SIGNAL("dtmf"), str(k))

  def updateDuration(self):
    elapsed=self.call.duration.elapsed()/1000
    h=elapsed/3600
    m=(elapsed-h*3600)/60
    s=(elapsed-h*3600-m*60)
    self.call_in_progress.duration.setText("%02d:%02d:%02d" % (h, m, s))

  def setStatus(self, status):
    if status=="active":
      self.duration_timer=QtCore.QTimer(self)
      self.connect(self.duration_timer, QtCore.SIGNAL("timeout()"), self.updateDuration)
      self.duration_timer.start(1000)
      self.setCurrentWidget(self.call_in_progress)

  def closeEvent(self, e):
    if self.duration_timer:
      self.duration_timer.stop()
    if self.in_call:
      self.emit(QtCore.SIGNAL("callEnded"), self.call)
