from PyQt4 import QtCore, QtGui, uic

from . import __path__

(Ui_ContactViewer, ContactViewerBaseClass) = uic.loadUiType(__path__[0]+"/ui/contact_viewer.ui")
(Ui_ContactEditor, ContactEditorBaseClass) = uic.loadUiType(__path__[0]+"/ui/contact_editor.ui")

class ContactViewer(ContactViewerBaseClass, Ui_ContactViewer):
  def __init__(self, parent):
    ContactViewerBaseClass.__init__(self, parent)
    self.setupUi(self)
    self.contact=None

  def setContact(self, contact):
    self.contact=contact
    self.name.setText(contact.name)
    self.phone_number.setText(contact.phone_number)

class ContactEditor(ContactEditorBaseClass, Ui_ContactEditor):
  def __init__(self, parent):
    ContactEditorBaseClass.__init__(self, parent)
    self.setupUi(self)
    self.contact=None

  def setContact(self, contact):
    self.contact=contact
    if self.contact:
      self.name.setText(contact.name)
      self.phone_number.setText(contact.phone_number)
    else:
      self.name.setText("")
      self.phone_number.setText("")

class Contacts(QtGui.QStackedWidget):
  class Entry():
    def __init__(self, row):
      for i in ("id", "name", "phone_number"):
        setattr(self, i, row[i])

  class Model(QtCore.QAbstractListModel):
    def __init__(self, parent, db):
      QtCore.QAbstractListModel.__init__(self, parent)

      self.db=db

      self.loadContacts()

    def loadContacts(self):
      query=self.selectContacts()
      
      self.contacts=list()
      for row in query:
        self.contacts.append(Contacts.Entry(row))
    
    def reload(self):
      self.loadContacts()
      self.reset()

    def selectContacts(self, where="", values=None):
      query=self.db.cursor()
      sql="select id, name, phone_number from contact "
      if where: sql+=where
      sql+="order by name"

      if values:
        query.execute(sql, values)
      else:
        query.execute(sql)
      return query

    def newCall(self, call):
      # reload the call log entry from the database, so that it is properly formatted as a row
      query=self.selectCalls("where date_time=?", (call.date_time.toTime_t(),))
      self.beginInsertRows(self.index(0, 0), 0, 0)
      self.calls.insert(0, CallLog.Entry(query.fetchone()))
      self.endInsertRows()

    def rowCount(self, parent):
      return len(self.contacts)

    def data(self, index, role):
      if role==QtCore.Qt.DisplayRole:
        return QtCore.QVariant(self.contacts[index.row()])
      else:
        return QtCore.QVariant()

  class ItemDelegate(QtGui.QAbstractItemDelegate):
    def __init__(self, parent):
      QtGui.QAbstractItemDelegate.__init__(self, parent)

    def paint(self, painter, option, index):
      contact=index.data().toPyObject()

      if option.state & QtGui.QStyle.State_Selected:
        painter.fillRect(option.rect, option.palette.highlight())
      else:
        painter.drawLine(option.rect.bottomLeft(), option.rect.bottomRight())

      # draw the contact name in big
      r=QtCore.QRect(option.rect)
      r.setWidth(r.width()/2)
      font=painter.font()
      font.setBold(True)
      old_point_size=font.pointSize()
      font.setPointSize(16)
      painter.drawText(r, QtCore.Qt.AlignHCenter|QtCore.Qt.AlignVCenter, contact.name)
      font.setPointSize(old_point_size)
      font.setBold(False)

      # draw the phone number in little
      r.translate(r.width(), 0)
      painter.drawText(r, QtCore.Qt.AlignHCenter|QtCore.Qt.AlignVCenter, contact.phone_number)

    def sizeHint(self, option, index):
      return QtCore.QSize(0, 40)

  class List(QtGui.QWidget):
    def __init__(self, parent):
      QtGui.QWidget.__init__(self, parent)
      self.list=QtGui.QListView(self)
      self.new_contact=QtGui.QPushButton(QtGui.QIcon(__path__[0]+"/icons/add_user.png"), "New contact...", self)
      self.new_contact.setIconSize(QtCore.QSize(32, 32))
      l=QtGui.QVBoxLayout(self)
      l.addWidget(self.list)
      l.addWidget(self.new_contact)

    def setModel(self, model):
      self.list.setModel(model)

    def setItemDelegate(self, item_delegate):
      self.list.setItemDelegate(item_delegate)

    def reload(self):
      self.list.model().reload()

  def __init__(self, parent, db, phone):
    QtGui.QStackedWidget.__init__(self, parent)

    self.db=db
    self.phone=phone

    self.list=Contacts.List(self)
    self.list.setModel(Contacts.Model(self, db))
    self.list.setItemDelegate(Contacts.ItemDelegate(self))
    self.connect(self.list.list, QtCore.SIGNAL("clicked(const QModelIndex &)"), self.onContactClicked)
    self.connect(self.list.new_contact, QtCore.SIGNAL("clicked()"), self.onNewContactClicked)

    self.contact_viewer=ContactViewer(self)
    self.connect(self.contact_viewer.edit, QtCore.SIGNAL("clicked()"), self.onEditContact)
    self.connect(self.contact_viewer.delete_, QtCore.SIGNAL("clicked()"), self.onDeleteContact)
    self.connect(self.contact_viewer.call, QtCore.SIGNAL("clicked()"), self.onCallContact)
    self.connect(self.contact_viewer.back, QtCore.SIGNAL("clicked()"), self.onBack)

    self.contact_editor=ContactEditor(self)
    self.connect(self.contact_editor.save, QtCore.SIGNAL("clicked()"), self.onSaveContact)
    self.connect(self.contact_editor.cancel, QtCore.SIGNAL("clicked()"), self.onCancelContact)

    self.addWidget(self.list)
    self.addWidget(self.contact_viewer)
    self.addWidget(self.contact_editor)
    self.setCurrentWidget(self.list)

  def onContactClicked(self, index):
    contact=index.data().toPyObject()
    self.contact_viewer.setContact(contact)
    self.setCurrentWidget(self.contact_viewer)

  def onNewContactClicked(self):
    self.emit(QtCore.SIGNAL("createContact"))

  def onEditContact(self):
    self.contact_editor.setContact(self.contact_viewer.contact)
    self.setCurrentWidget(self.contact_editor)

  def onDeleteContact(self):
    contact=self.contact_viewer.contact
    #d=QtGui.QMessageBox(QtGui.QMessageBox.Question, "Delete a contact", 
    #    "Delete the contact named\n\""+contact.name+"\" ?", 
    #    QtGui.QMessageBox.Ok|QtGui.QMessageBox.Cancel)
    if True: #self.phone.desktop.dialog(self.phone, d)==QtGui.QMessageBox.Ok:
      self.db.cursor().execute("delete from contact where id=?", (contact.id,))
      self.list.reload()
      self.emit(QtCore.SIGNAL("contactModified"))
      self.setCurrentWidget(self.list)

  def onCallContact(self):
    contact=self.contact_viewer.contact
    self.emit(QtCore.SIGNAL("callContact"), str(contact.phone_number), contact.name)

  def onBack(self):
    self.setCurrentWidget(self.list)

  def onSaveContact(self):
    contact=self.contact_editor.contact
    query=self.db.cursor()
    if contact:
      n=self.contact_editor.name.text()
      contact.name=n
      contact.phone_number=str(self.contact_editor.phone_number.text())
      query.execute("update contact set name=?, phone_number=? where id=?", (str(n.toUtf8()), contact.phone_number, contact.id))
      self.contact_viewer.setContact(contact)
      self.setCurrentWidget(self.contact_viewer)
    else:
      query.execute("insert into contact(id, name, phone_number) values(NULL, ?, ?)", (str(self.contact_editor.name.text().toUtf8()), str(self.contact_editor.phone_number.text().toUtf8())))
      self.setCurrentWidget(self.list)

    self.list.reload()
    self.emit(QtCore.SIGNAL("contactModified"))
    

  def onCancelContact(self):
    if self.contact_editor.contact:
      self.setCurrentWidget(self.contact_viewer)
    else:
      self.setCurrentWidget(self.list)

  def createContact(self, phone_number):
    # slightly hacky
    self.contact_editor.setContact(None)
    self.contact_editor.phone_number.setText(phone_number)
    self.setCurrentWidget(self.contact_editor)

