import os
from PyQt4 import QtCore, QtGui

from . import __path__
from core.auimd_widget import *

class Notes(QtGui.QWidget, AuimdWidget):
  @staticmethod
  def defaultWidgetIcon():
    return QtGui.QIcon(__path__[0]+"/icons/kontact_notes.png")

  @staticmethod
  def defaultWidgetTitle():
    return "Notes"

  @staticmethod
  def getCategory():
    return "PIM"

  def __init__(self, parent, *args):
    QtGui.QWidget.__init__(self, parent)
    AuimdWidget.__init__(self, parent, args)

    self.filename=None

    #self.setAutoFillBackground(1)

    layout=QtGui.QVBoxLayout(self)
    self.toolbar=QtGui.QToolBar(self)
    self.toolbar.setIconSize(QtCore.QSize(16,16))
    #self.toolbar.setOrientation(QtCore.Qt.Vertical)
    QtCore.QObject.connect(self.toolbar, QtCore.SIGNAL("actionTriggered(QAction*)"), self.toolbarAction)

    self.text_edit=QtGui.QTextEdit(self)
    self.default_point_size=12.0
    self.text_edit.setFontPointSize(self.default_point_size)

    self.action_cut=self.toolbar.addAction(QtGui.QIcon(__path__[0]+"/icons/editcut.png"), "cut", self.text_edit.cut)
    self.action_copy=self.toolbar.addAction(QtGui.QIcon(__path__[0]+"/icons/editcopy.png"), "copy", self.text_edit.copy)
    self.action_paste=self.toolbar.addAction(QtGui.QIcon(__path__[0]+"/icons/editpaste.png"), "paste", self.text_edit.paste)
    self.toolbar.addSeparator()
    self.action_undo=self.toolbar.addAction(QtGui.QIcon(__path__[0]+"/icons/undo.png"), "undo", self.text_edit.undo)
    self.action_redo=self.toolbar.addAction(QtGui.QIcon(__path__[0]+"/icons/redo.png"), "redo", self.text_edit.redo)
    self.toolbar.addSeparator()
    self.action_bold=self.toolbar.addAction(QtGui.QIcon(__path__[0]+"/icons/text_bold.png"), "bold")
    self.action_bold.setCheckable(1);
    self.action_italic=self.toolbar.addAction(QtGui.QIcon(__path__[0]+"/icons/text_italic.png"), "italic")
    self.action_italic.setCheckable(1);
    self.action_under=self.toolbar.addAction(QtGui.QIcon(__path__[0]+"/icons/text_under.png"), "under")
    self.action_under.setCheckable(1);
    self.toolbar.addSeparator()
    self.action_mag_inc=self.toolbar.addAction(QtGui.QIcon(__path__[0]+"/icons/viewmag+.png"), "increase font")
    self.action_mag_dec=self.toolbar.addAction(QtGui.QIcon(__path__[0]+"/icons/viewmag-.png"), "decrease font")

    layout.addWidget(self.toolbar)
    layout.addWidget(self.text_edit)

    self.left_button.setText("Menu")
    self.right_button.setText("Close")

    m=QtGui.QMenu(self.left_button)
    self.action_new=QtGui.QAction(QtGui.QIcon(__path__[0]+"/icons/filenew.png"), "New", m)
    self.action_open=QtGui.QAction(QtGui.QIcon(__path__[0]+"/icons/fileopen.png"), "Open...", m)
    self.action_save=QtGui.QAction(QtGui.QIcon(__path__[0]+"/icons/filesave.png"), "Save", m)
    self.action_save_as=QtGui.QAction(QtGui.QIcon(__path__[0]+"/icons/filesaveas.png"), "Save as...", m)
    m.addAction(self.action_new)
    m.addAction(self.action_open)
    m.addSeparator()
    m.addAction(self.action_save)
    m.addAction(self.action_save_as)
    self.left_button.setMenu(m)
    QtCore.QObject.connect(m, QtCore.SIGNAL("triggered(QAction*)"), self.menuAction)

    if args and len(args):
      self.openFile(args[0])

  def openFile(self, filename):
    try:
      self.text_edit.setText(open(filename, "r").read())
      self.filename=filename
      self.updateTitle()
    except:
      QtGui.QMessageBox.critical(self, "Error", "The file "+filename+" could not be opened")

  def updateTitle(self):
    title=self.defaultWidgetTitle()

    if self.filename:
      title+=" - "+os.path.basename(str(self.filename))

    self.setWidgetTitle(title)

  def saveFile(self, filename):
    try:
      open(filename, "w").write(self.text_edit.toHtml())
      self.filename=filename
      self.updateTitle()
    except:
      QtGui.QMessageBox.critical(self, "Error", "The file "+filename+" could not be saved")

  def rightButtonAction(self):
    self.desktop.closeWidget(self)

  def menuAction(self, action):
    if action is self.action_save:
      if self.filename:
        filename=self.filename
      else:
        filename=self.desktop.saveFile(self)
      if filename:
        self.saveFile(filename)

    elif action is self.action_save_as:
      filename=self.desktop.saveFile(self)
      if filename:
        self.saveFile(filename)

    elif action is self.action_open:
      filename=self.desktop.openFile(self)
      if filename:
        self.openFile(filename)

    elif action is self.action_new:
      self.text_edit.clear()
      self.filename=None
      self.updateTitle()

  def toolbarAction(self, action):
    if action is self.action_bold:
      if self.text_edit.fontWeight()==QtGui.QFont.Bold:
        self.text_edit.setFontWeight(QtGui.QFont.Normal)
        action.setChecked(0)
      else:
        self.text_edit.setFontWeight(QtGui.QFont.Bold)
        action.setChecked(1)

    elif action is self.action_italic:
      if self.text_edit.fontItalic():
        self.text_edit.setFontItalic(0)
        action.setChecked(0)
      else:
        self.text_edit.setFontItalic(1)
        action.setChecked(1)

    elif action is self.action_under:
      if self.text_edit.fontUnderline():
        self.text_edit.setFontUnderline(0)
        action.setChecked(0)
      else:
        self.text_edit.setFontUnderline(1)
        action.setChecked(1)

    elif action is self.action_mag_inc:
      f=self.text_edit.fontPointSize()
      if f==0: f=self.default_point_size
      self.text_edit.setFontPointSize(f+2)
    elif action is self.action_mag_dec:
      f=self.text_edit.fontPointSize()
      if f>2: self.text_edit.setFontPointSize(f-2)
