import os
from PyQt4 import QtCore, QtGui, uic

from . import __path__
from core.auimd_widget import *
from core.xembed_widget import *

class Xephyr(QtGui.QStackedWidget, AuimdWidget):
  @staticmethod
  def defaultWidgetIcon():
    return QtGui.QIcon(__path__[0]+"/icons/window_nofullscreen.png")

  @staticmethod
  def defaultWidgetTitle():
    return "Xephyr"

  @staticmethod
  def getCategory():
    return "Tools"

  def __init__(self, parent):
    QtGui.QStackedWidget.__init__(self, parent)
    AuimdWidget.__init__(self, parent)

    self.embeded=None

    self.settings=uic.loadUi(__path__[0]+"/ui/settings.ui")
    self.addWidget(self.settings)

    # not really safe but best than nothing anyway
    for d in range(0,9):
      if not os.path.exists("/tmp/.X11-unix/X%d" % d):
        self.settings.display.addItem(str(d))
    
    self.connect(self.settings.start, QtCore.SIGNAL("clicked(bool)"), self.startXephyr)

    self.left_button.setText("")
    self.right_button.setText("Close")
  
  def startXephyr(self):
    # base arguments
    args=["-parent", "<winid>"]

    # geometry
    i=self.settings.size.currentIndex()
    if i==1:
      w=240
      h=320
    elif i==2:
      w=640
      h=480
    elif i==3:
      w=1024
      h=768
    elif i==4:
      w=1280
      h=1024
    else:
      w="<width>"
      h="<height>"

    if i!=0:
      self.desktop.setCurrentWidgetVirtualSize(QtCore.QSize(w, h))

    # depth
    i=self.settings.depth.currentIndex()
    if i==0:
      d="8"
    elif i==1:
      d="16"
    else:
      d="24"
    
    args.extend(("-screen", str(w)+"x"+str(h)+"/"+d))

    # access control
    if self.settings.no_ac.isChecked():
      args.append("-ac")
    
    # add display number
    display=str(":"+self.settings.display.currentText())
    args.append(display)
    self.setWidgetTitle(self.defaultWidgetTitle()+" - "+display)

    self.embeded=XEmbedWidget(self, "Xephyr", "Xephyr", args)
    self.connect(self.embeded, QtCore.SIGNAL("finished()"), self.finished)
    self.addWidget(self.embeded)
    self.setCurrentWidget(self.embeded)

  def finished(self):
    self.desktop.closeWidget(self)

  def rightButtonAction(self):
    if self.embeded:
      self.embeded.close()
    self.desktop.closeWidget(self)

  def closeEvent(self, e):
    self.embeded.close()
